/*
 * Copyright (c) 2020 Nordic Semiconductor ASA
 *
 * SPDX-License-Identifier: LicenseRef-BSD-5-Clause-Nordic
 */
#include <zephyr.h>
#include <lte_lc.h>
#include <net/cloud.h>
#include <net/socket.h>

static struct cloud_backend *cloud_backend;

static int ready = 0;
static int disconnected = 0;

void cloud_event_handler(const struct cloud_backend *const backend,
			 const struct cloud_event *const evt,
			 void *user_data)
{
	ARG_UNUSED(user_data);
	ARG_UNUSED(backend);

	switch (evt->type) {
	case CLOUD_EVT_CONNECTED:
		printk("CLOUD_EVT_CONNECTED\n");
		break;
	case CLOUD_EVT_READY:
		printk("CLOUD_EVT_READY\n");
        ready = 1;
		break;
	case CLOUD_EVT_DISCONNECTED:
		printk("CLOUD_EVT_DISCONNECTED\n");
        disconnected = 1;
		break;
	case CLOUD_EVT_ERROR:
		printk("CLOUD_EVT_ERROR\n");
		break;
	case CLOUD_EVT_DATA_SENT:
		printk("CLOUD_EVT_DATA_SENT\n");
		break;
	case CLOUD_EVT_DATA_RECEIVED:
		printk("CLOUD_EVT_DATA_RECEIVED\n");
		printk("Data received from cloud: %s\n", evt->data.msg.buf);
		break;
	case CLOUD_EVT_PAIR_REQUEST:
		printk("CLOUD_EVT_PAIR_REQUEST\n");
		break;
	case CLOUD_EVT_PAIR_DONE:
		printk("CLOUD_EVT_PAIR_DONE\n");
		break;
	case CLOUD_EVT_FOTA_DONE:
		printk("CLOUD_EVT_FOTA_DONE\n");
		break;
	default:
		printk("Unknown cloud event type: %d\n", evt->type);
		break;
	}
}

static void modem_configure(void)
{
#if defined(CONFIG_BSD_LIBRARY)
	if (IS_ENABLED(CONFIG_LTE_AUTO_INIT_AND_CONNECT)) {
		/* Do nothing, modem is already turned on
		 * and connected.
		 */
	} else {
		int err;

		printk("Connecting to LTE network. ");
		printk("This may take several minutes.\n");

		err = lte_lc_init_and_connect();
		if (err) {
			printk("LTE link could not be established.\n");
		}
        else {
		    printk("Connected to LTE network\n");
        }

#if defined(CONFIG_POWER_SAVING_MODE_ENABLE)
		err = lte_lc_psm_req(true);
		if (err) {
			printk("lte_lc_psm_req, error: %d\n", err);
		}

		printk("PSM mode requested\n");
#endif
	}
#endif
}

static int cloud_socket_poll(void) 
{
	int err;

    struct pollfd fds[] = {
		{
			.fd = cloud_backend->config->socket,
			.events = POLLIN
		}
	};

	err = poll(fds, ARRAY_SIZE(fds), cloud_keepalive_time_left(cloud_backend));
    if (err < 0) {
        printk("poll() returned an error: %d\n", err);
        return err;
    }

    if (err == 0) {
        return -1; //timeout is error for our case.
    }

    if ((fds[0].revents & POLLIN) == POLLIN) {
        cloud_input(cloud_backend);
        return 0;
    }

    if ((fds[0].revents & POLLNVAL) == POLLNVAL) {
        printk("Socket error: POLLNVAL\n");
        printk("The cloud socket was unexpectedly closed.\n");
        return -1;
    }

    if ((fds[0].revents & POLLHUP) == POLLHUP) {
        printk("Socket error: POLLHUP\n");
        printk("Connection was closed by the cloud.\n");
        return -1;
    }

    if ((fds[0].revents & POLLERR) == POLLERR) {
        printk("Socket error: POLLERR\n");
        printk("Cloud connection was unexpectedly closed.\n");
        return -1;
    }
	

    return -1;
}

void main(void)
{
	int err;
    int i;

	printk("Cloud client has started\n");

	cloud_backend = cloud_get_binding(CONFIG_CLOUD_BACKEND);
	__ASSERT(cloud_backend != NULL, "%s backend not found",
		 CONFIG_CLOUD_BACKEND);

	err = cloud_init(cloud_backend, cloud_event_handler);
	if (err) {
		printk("Cloud backend could not be initialized, error: %d\n",
			err);
        goto err0;
	}

	modem_configure();

    for (i = 0; i < 2; i++) {
        printk("Connecting to Cloud...\n");
        
        ready = 0;
        disconnected = 0;

        err = cloud_connect(cloud_backend);
        if (err) {
            printk("cloud_connect, error: %d\n", err);
            goto err0;
        }

        //wait for connection
        while (ready == 0) {
            err = cloud_socket_poll();
            if (err < 0) {
                printk("cloud_socket_poll, error: %d\n", err);
                goto err0;
            }
        }

        printk("Disconnecting from Cloud.\n");

        err = cloud_disconnect(cloud_backend);
        if (err) {
            printk("cloud_disconnect, error: %d\n", err);
            goto err0;
        }

        //wait for disconnection.
        while (disconnected == 0) {
            err = cloud_socket_poll();
            if (err < 0) {
                printk("cloud_socket_poll, error: %d\n", err);
                goto err0;
            }
        }

        printk("Sleeping...\n");
        k_sleep(K_SECONDS(10));
    }

err0:
    printk("Done.\n");
    while(1);
}
